from hygia.data_pipeline.feature_engineering.key_smash import KeySmash
from hygia.data_pipeline.feature_engineering.word_embedding import WordEmbedding
import pandas as pd

class FeatureEngineering:
    """
    A class for extracting key smashing and word embedding features from text data.
    This class combines the functionality of the KeySmash and WordEmbedding classes
    to extract key smashing and word embedding features from a given text column in a dataframe.

    Examples
    --------
    Use this class like this:

    .. code-block:: python

        feature_engineer = FeatureEngineering()
        df = feature_engineer.extract_features(df, "text_column")
        print(df)
    """

    def __init__(self, lang: str = 'es', dimensions: int = 25, model: str = 'bytepair'):
        """
        Initialize the FeatureEngineering class.
        
        :param lang: The language of the text to be processed. (default 'es')
        :type lang: str
        :param dimensions: The number of dimensions of the word embedding vectors. (default 25)
        :type dimensions: int
        :param model: The word embedding model to be used. (default 'bytepair')
        :type model: str
        """
        self.key_smash = KeySmash()
        self.word_embedding = WordEmbedding(lang=lang, dimensions=dimensions, model=model)

    def extract_features(self, df: pd.DataFrame, text_column: str) -> pd.DataFrame:
        """
        Extract key smashing and word embedding features from a given dataframe and column.

        :param df: Dataframe to extract features from.
        :type df: pandas.DataFrame
        :param text_column: Name of the column in the dataframe that contains the text data to extract features from.
        :type text_column: str
        :param normalize: Indicates whether to normalize the feature columns. Default is True.
        :type normalize: bool, optional

        :return: The input dataframe with additional columns for key smashing and word embedding features.
        :rtype: pandas.DataFrame

        Examples
        --------
        Use this class like this:

        .. code-block:: python

            fe = FeatureEngineering()
            df = fe.extract_features(df, "text_column")
            print(df)
        """
        df = self.key_smash.extract_key_smash_features(df, text_column)
        df = self.word_embedding.extract_word_embedding_features(df, text_column)
        return df



from hygia.parser.parser_base import ParserBase

class PreProcessingParser(ParserBase):
    
    def __init__(self, columns_name):
        self.columns_name = columns_name
        
    def parse(self, data: list):
        return self._parse_pre_processing_configs(data)
        
    def _parse_pre_processing_configs(self, data: list):
        if(not data): return
        
        aliases = self._try_get(data, 'aliases')
        self._get_dataframe(aliases)
        
        return aliases, self.columns_name
    
    def _get_dataframe(self, aliases: dict):
        if(not aliases): return 
        
        for alias in aliases:
            for key in alias.keys():
                if(not self._has_column_on_df(self.columns_name, alias)):
                    self.columns_name.append(key)
                



import pandas as pd

class PreProcessData:
    def concatenate_columns(self, df, columns, concatenated_column_name):
        df[concatenated_column_name] = df[columns].astype(str).agg(' '.join, axis=1)
        df[concatenated_column_name] = df[concatenated_column_name].str.replace(r'  +', 'EMPTY', regex=True).astype('str')
        return df
    
    def pre_process_data(self, df, columns_to_concat, concatenated_column_name):
        df = self.concatenate_columns(df, columns_to_concat, concatenated_column_name)
        return df




from hygia.data_pipeline.feature_engineering.feature_engineering import (FeatureEngineering)
from hygia.data_pipeline.feature_engineering.key_smash import (KeySmash)
from hygia.data_pipeline.feature_engineering.word_embedding import (WordEmbedding)
from hygia.data_pipeline.model.random_forest import (RandomForestModel)
from hygia.data_pipeline.pre_process_data.pre_process_data import (PreProcessData)
from hygia.main import (run_with_config)

__all__ = [
    "FeatureEngineering",
    "KeySmash",
    "WordEmbedding",
    "RandomForestModel",
    "PreProcessData",
    "run_with_config"
]


import os
import pandas as pd

from hygia import PreProcessData
from hygia import FeatureEngineering
from hygia import RandomForestModel

from hygia.parser.model_parser import ModelParser
from hygia.parser.YAML_parser import YAMLParser
from hygia.parser.pre_processing_parser import PreProcessingParser
from hygia.parser.feature_engineering_parser import FeatureEngineeringParser
    
def run_with_config(yaml_path: str):
    initialParser = YAMLParser
    
    preProcessingParser = PreProcessingParser
    preProcessData = PreProcessData
    
    featureEngineeringParser = FeatureEngineeringParser
    featureEngineering = FeatureEngineering
    
    modelParser = ModelParser
    randomForestModel = RandomForestModel

    config = initialParser(yaml_path).parse()
    
    for data in config['data_path']:
        # Load csv
        separator = config['separator']
        encoding = config['encoding']
        engine = config['engine']
        nrows = config['nrows']
        df = pd.read_csv(data, sep=separator, engine=engine, encoding=encoding, nrows=nrows)
        
        # Pre processing
        columns_name = list(df.columns)
        
        columns_set, columns_name = preProcessingParser(columns_name).parse(config['pre_processing'])
        for columns in columns_set:       
            for key, value in columns.items():
                df = preProcessData().pre_process_data(df, value, key)
        
        # Feature engineering
        features_configs, columns_alias = featureEngineeringParser(columns_name).parse(config['feature_engineering'])
        for feature_config in features_configs:
            feature_columns = feature_config['columns']
            
            for column in feature_columns:
                lang = feature_config['data_lang']
                dimensions = feature_config['dimensions'][column]
                df = featureEngineering(lang=lang, dimensions=dimensions).extract_features(df, column)
                
        # Model
        model_configs = modelParser(columns_alias).parse(config['model'])
        for model_config in model_configs:
            model_columns = model_config['columns']
            trained_model_file = model_config['trained_model_file']
            
            for column in model_columns:
                featured_df = df.loc[:, df.columns.str.endswith(column)]
                df[f'prediction_{column}'] = randomForestModel(model_file=trained_model_file).predict(featured_df.iloc[:,-30:])
            
        del config['pre_processing']
        del config['feature_engineering']
        del config['model']
        
        return df.loc[:, df.columns.str.startswith('prediction_')]
         


import yaml

from hygia.parser.parser_base import ParserBase

class YAMLParser(ParserBase):

    def parse(self):
        return self._parse_yaml()

    def _parse_yaml(self):
        with open(self.filepath, 'r') as file:
            dag_config_dict = yaml.safe_load(file)

        dag = self._try_get(dag_config_dict, 'dag')
        data_config = self._try_get(dag, 'data_config')
        
        separator = self._get(data_config, 'separator', '¨')
        engine = self._get(data_config, 'engine', 'python')
        encoding = self._get(data_config, 'encoding', 'utf-8')
        nrows = self._get(data_config, 'nrows', 10000)

        initial_parser = {
            "nrows": nrows,
            "engine": engine,
            "encoding": encoding,
            "separator": separator,
            "dag_id": self._try_get(dag, 'id'),
            "model": self._try_get(dag, 'model'),
            "data_path": self._try_get(dag, 'data_path'),
            "description": self._try_get(dag, 'description'),
            "output_folder": self._try_get(dag, 'output_folder'),
            "pre_processing": self._try_get(dag, 'pre_processing'),
            "feature_engineering": self._try_get(dag, 'feature_engineering'),
        }

        return initial_parser



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hygia',
 'hygia.data_pipeline',
 'hygia.data_pipeline.feature_engineering',
 'hygia.data_pipeline.model',
 'hygia.data_pipeline.pre_process_data',
 'hygia.parser']

package_data = \
{'': ['*']}

install_requires = \
['altair==4.2.0',
 'attrs==22.2.0',
 'bpemb==0.3.4',
 'certifi==2022.12.7',
 'charset-normalizer==3.0.1',
 'contourpy==1.0.6',
 'coverage==7.0.2',
 'cycler==0.11.0',
 'entrypoints==0.4',
 'exceptiongroup==1.1.0',
 'fonttools==4.38.0',
 'gensim==3.8.3',
 'idna==3.4',
 'importlib-resources==5.10.2',
 'iniconfig==1.1.1',
 'jinja2==3.1.2',
 'joblib==1.2.0',
 'jsonschema==4.17.3',
 'kiwisolver==1.4.4',
 'markupsafe==2.1.1',
 'matplotlib==3.6.2',
 'numpy==1.24.1',
 'packaging==23.0',
 'pandas==1.5.2',
 'pillow==9.4.0',
 'pkgutil-resolve-name==1.3.10',
 'pluggy==1.0.0',
 'pyparsing==3.0.9',
 'pyrsistent==0.19.3',
 'pytest-cov==4.0.0',
 'pytest==7.2.0',
 'python-dateutil==2.8.2',
 'pytz==2022.7',
 'pyyaml==6.0',
 'requests==2.28.2',
 'scikit-learn==1.2.0',
 'scipy==1.9.3',
 'sentencepiece==0.1.97',
 'six==1.16.0',
 'smart-open==6.3.0',
 'threadpoolctl==3.1.0',
 'tomli==2.0.1',
 'toolz==0.12.0',
 'tqdm==4.64.1',
 'urllib3==1.26.14',
 'whatlies==0.7.0',
 'wheel==0.38.4',
 'zipp==3.11.0']

setup_kwargs = {
    'name': 'hygia',
    'version': '0.1.8',
    'description': 'A short description of the package.',
    'long_description': '<p align="center">\n    <img src="./assets/img/horizontal_logo.PNG" alt="hygia-logo" style="width:500px;"/>\n</p>\n\n# A powerful Python ML playground toolkit\n\n[![PyPI Latest Release](https://img.shields.io/pypi/v/hygia.svg)](https://pypi.org/project/hygia/)\n[![License](https://img.shields.io/pypi/l/hygia.svg)](https://github.com/hygia-org/hygia/blob/main/LICENSE)\n[![Coverage](https://codecov.io/github/hygia-org/hygia/coverage.svg?branch=main)](https://codecov.io/gh/hygia-org/hygia)\n\n<!-- [![Package Status](https://img.shields.io/pypi/status/hygia.svg)](https://pypi.org/project/hygia/) -->\n\n## What is it?\n\nHygia is a Python package that provides fast, flexible, and expressive data pipeline configuration through a YAML file to make working with Machine Learning data easy and intuitive. It consists of helping developers and users to register, organize, compare and share all their ML model metadata in a single place, facilitating the generation of requirements in the ETL (Extract, Transform and Load) process. Thus, the migration can be scaled, automated, and accelerated for similar contexts.\n\n## Main Features\n\n- Configure data pipeline through a YAML file\n- Execute through command line or python import\n- Pack the solution into a Python\'s Package Manager\n- Visualize results in customized dashboards\n- Test on different databases\n\n## Where to get it\n\nThe source code is currently hosted on GitHub at: `https://github.com/hygia-org`\n\n## Installation from sources\n\n```\npython -m venv env\nsource env/bin/activate\npip install -r requirements.txt\n```\n\n### Boilerplate\n\n```\nexamples/hygia_boilerplate.ipynb\n```\n\n### Testing\n\n```\npytest --cov\n```\n\n### Documentation\n\nWe used sphinx to write the documentation\n\nTo run locally, you need to install sphinx:\n\n```\npip install sphinx\n```\n\nThen install the theme used:\n\n```\npip install pydata-sphinx-theme\n```\n\nAnd Run the project\n\n```\nsphinx-build -b html source ./\n```\n\nAnd open the index.html\n',
    'author': 'PDA-FGA',
    'author_email': 'rocha.carla@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/PDA-FGA/Playground',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
